use anyhow::Result;
use bidir::{
  bidir_debruijn::{app_ctx, synthesize},
  data_debruijn::convert_term,
  data_debruijn::Context,
  parser::TermParser,
};
use rustyline::{Config, DefaultEditor};

fn main() -> Result<()> {
  let term_parser = TermParser::new();

  let rl_config = Config::builder().auto_add_history(true).build();
  let mut rl = DefaultEditor::with_config(rl_config)?;

  loop {
    let line = match rl.readline(">> ") {
      Ok(line) => line,
      Err(_) => break,
    };

    // let line = r"\x.\y.x";

    let parsed_term = match term_parser.parse(&line) {
      Ok(term) => term,
      Err(err) => {
        eprintln!("parser error: {err}");
        continue;
      }
    };
    println!("parsed: {:?}", parsed_term);

    let converted_term = convert_term(&parsed_term);
    println!("converted: {converted_term:?}");

    let ctx = Context::default();
    let (ty, out_ctx) = match synthesize(&ctx, &converted_term) {
      Ok(v) => v,
      Err(err) => {
        eprintln!("typecheck error: {err}");
        eprintln!("{}", err.backtrace());
        continue;
      }
    };

    let ty = match app_ctx(&out_ctx, &ty) {
      Ok(v) => v,
      Err(err) => {
        eprintln!("substitution error: {err}");
        eprintln!("{}", err.backtrace());
        continue;
      }
    };

    println!("synthesized: {:?}", ty);
    // break;
  }

  Ok(())
}
